***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### Eab2DFT ###

c     Evaluates the off-diagonal Lagrange multipliers in case of a local
c     exchange approximation

      subroutine Eab2DFT(iorb1,iorb2,psi,pot,excp,wgt2,wk0,wk1)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)

      include 'commons8.inc'
      dimension psi(*),pot(*),excp(*),wgt2(*),wk0(*),wk1(*)

      if (mgx(6,iorb1).ne.mgx(6,iorb2)) return
      if (ige(iorb1).ne.ige(iorb2)) return

c FIXME
      ipc12=iorb1+(iorb2-1)*norb
      ipc21=iorb2+(iorb1-1)*norb
      engo(ipc12)=zero

      iborb1=i1b(iorb1)
      isiorb1=i1si(iorb1)	
      iborb2=i1b(iorb2)
      isiorb2=i1si(iorb2)	

c     Add contributions from the Coulomb and local exchange potentials.
c     In the local exchange approximtion the Coulomb potential includes
c     also the contribution from a given orbital

c     Calculate the Coulomb potential contribution from all the orbitals

      do i=1,mxsize
         wk0(i)=0.d0
      enddo
	
      do iorb=1,norb
         iborb=i1b(iorb)
         ibpot=i2b(iorb)	
         nmut=i1mu(iorb)
         isipot=i2si(iorb)
         oc=occ(iorb)
         call axpy (isipot,oc,pot(ibpot),ione,wk0,ione)
      enddo

      call prod  (isiorb2,psi(iborb2),wk0)

c     Multiply the local exchange potential by psi(iborb2) 
c     and add the result to the Coulomb potential

      call prod2 (isiorb2,psi(iborb2),excp(length-mxsize),wk1)
      call add (isiorb2,wk0,wk1)

c     To complete the integrand wk1 has to be multiplied by psi(iborb1)

      ngrid=min(isiorb1,isiorb2)
      call prod (ngrid,psi(iborb1),wk1)
      wtwoel=dot(ngrid,wgt2,ione,wk1,ione)

      engo(ipc1)= wtwoel
      engo(ipc2)= wtwoel

c FIXME
c        Damping factors are set to 0 by default (obsolete).

      engo(ipc1)=sflagra*((1.d0-dflagra)*engo1+dflagra*engoprv1)
      engo(ipc2)=sflagra*((1.d0-dflagra)*engo2+dflagra*engoprv2)

      if (iprint(48).ne.0) then
         write(*,'(a8,i4,e16.6,i4,a8,i4,a8,2e16.8)') 'Eab2DFT: ',
     &        lmtype,sflagra,
     &        iorn(iorb),bond(iorb1),iorn(iorb1),bond(iorb1),
     &        engo(ipc1),engo(ipc2)
      endif

      return
      end
