***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### blas ###
c
c     This is simplified replacement for BLAS routines: dcopy, daxpy, dscal,
c     ddot, mxv. Note that ix and iy are ignored and are set to 1.

      subroutine  copy(n,dx,ix,dy,iy)
      integer*4 i,ix,iy,n
      real*8  dx(*),dy(*)

      do i=1,n
         dy(i)=dx(i)
      enddo
      return
      end

      subroutine  axpy(n,da,dx,ix,dy,iy)
      integer*4  i,ix,iy,n
      real*8 da,dx(*),dy(*)

      do i=1,n
         dy(i)=da*dx(i)+dy(i)
      enddo
      return
      end

      subroutine  scal(n,da,dx,ix)
      integer*4 i,ix,n
      real*8 da,dx(*)

      do i=1,n
         dx(i)=da*dx(i)
      enddo
      return
      end

      real*8 function  dot(n,dx,ix,dy,iy)
      integer*4  i,ix,iy,n
      real*8  dx(*),dy(*),ddot

      dot=0.d0
      do i=1,n
         dot=dot+dx(i)*dy(i)
      enddo
      return
      end


c$$$c     Multiplies the matrix DX(nr,nc) times the vector DV(nc) and stores 
c$$$c     the result in the vector DY(nr) by means of calling DGEMV 
c$$$c
c$$$      subroutine dgemv (trans,nr1,nc,alpha,dx,nr2,dv,incr1,beta,
c$$$     &     dvr,incr2)
c$$$      integer*4  incr1,incr2,inr,nc,nr1,nr2
c$$$      real*8  alpha,beta,dx(nr1,nc),dv(nc),dvr(nr1),s
c$$$      character*1 trans
c$$$
c$$$      do inr=1,nr1
c$$$         s=0.d0
c$$$         do inc=1,nc
c$$$            s=s+dx(inr,inc)*dv(inc)
c$$$         enddo
c$$$         dvr(inr)=s
c$$$      enddo
c$$$
c$$$      return
c$$$      end
c$$$
      
c     Multiplies the matrix DX(nr,nc) times the vector DV(nc) and stores                                    
c     the result in the vector DY(nr) (simplified version of DGEMV)                                         
c                                                                                                           
      subroutine gemv (nr1,nc,dx,dv,dvr)
      integer*4  inr,inc,nc,nr1
      real*8  dx(nr1,*),dv(*),dvr(*),s

      do inr=1,nr1
         s=0.d0
         do inc=1,nc
            s=s+dx(inr,inc)*dv(inc)
         enddo
         dvr(inr)=s
      enddo



      return
      end



                      












