***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### diffnu ###

c    This routine calculates 

c     (\frac{\partial^2}{\partial\ni^2} + 
c           d(\ni,\mu) \frac{\partial}{\partial \ni}) f(\ni,\mu)

c     The function is imersed in the array f(nni+8,nmut+8)
c     To allow for the differentiation over ni variable in the form
c     of matrix times vector the array f has to be transposed first.
c     Results of differentiation are stored directly column-wise in 
c     another matrix which means doing back tranposition before 
c     returning to a calling routine.

c     This routine differentiates a function over ni variable.
c     the function is imersed in the array f(nni+6,nmut+6)

      subroutine diffnu (n,f,fd,wtran1,wtran2)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      data n9/9/
      dimension f(nni+8,n+8),fd(nni+8,n+8),
     &          wtran1(n+8,nni+8),wtran2(n+8,nni+8)

c     To allow for the differentiation over ni variable in the form
c     of matrix times vector the array f has to be transposed first.
c     Results of differentiation will be stored directly
c     column-wise in another matrix which means doing back tranposition
c     before returning to calling routine.  

      nni8=nni+8
      n8=n+8
      call gmtran (f,wtran1,nni8,n8)

c     FC3, ifort: error while passing 9 to a subroutine 

      do j=1,nni
c         call mxv (wtran1(1,j),n8,dni(1,j),n9,wtran2(1,j+4))
         call gemv (n8,n9,wtran1(1,j),dni(1,j),wtran2(1,j+4))
      enddo

      call gmtran(wtran2,fd,n8,nni8)

      return
      end

      subroutine diff1nu (n,f,fd,wtran1,wtran2)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      data n9/9/
      dimension f(nni+8,n+8),fd(nni+8,n+8),
     &          wtran1(n+8,nni+8),wtran2(n+8,nni+8)

c     To allow for the differentiation over ni variable in the form
c     of matrix times vector the array f has to be transposed first.
c     Results of differentiation will be stored directly
c     column-wise in another matrix which means doing back tranposition
c     before returning to calling routine.  

      nni8=nni+8
      n8=n+8

      call gmtran (f,wtran1,nni8,n8)

c     FC3, ifort: error while passing 9 to a subroutine 

      do j=1,nni
c         call mxv (wtran1(1,j),n8,d1ni(1,j),n9,wtran2(1,j+4))
         call gemv (n8,n9,wtran1(1,j),d1ni(1,j),wtran2(1,j+4))

      enddo

      call gmtran(wtran2,fd,n8,nni8)

      return
      end


