***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### etotalDFT ###

c     Calculates total energy using several DFT functionals

      subroutine etotalDFT (psi,pot,excp,e,f0,wgt1,wgt2,
     &     wk0,wk1,wk2,wk3,rhot,rhotup,rhotdown,
     &     grhot,grhotup,grhotdown,wk10,wk11,wk12,wk13)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
c      integer*4 rhot,rhotup,rhotdown,grhot,grhotup,grhotdown,wk4,wk5,
c     &     wk6,wk7
      parameter (const13=1.d0/3.d0,const43=4.d0/3.d0)
      include 'commons8.inc'	

      dimension psi(*),pot(*),excp(*),e(*),f0(*),wgt1(*),wgt2(*),
     &     wk0(*),wk1(*),wk2(*),wk3(*),rhot(*),rhotup(*),rhotdown(*),
     &     grhot(*),grhotup(*),grhotdown(*),
     &     wk10(*),wk11(*),wk12(*),wk13(*)

      if (iprint(75).ne.0) then
c        test n2f and nfnf routines
         print *,'Testing n2f'
         call testn2f(rhot,wk0,wk1,wk2,wk3)

         print *,' '
         print *,'Testing nfng'
         call testnfng(rhot,rhotup,rhotdown,grhotdown,grhotup,
     &        wk0,wk1,wk2,wk3,wk10)

         call n2f(psi,wk0,wk1,wk2,wk3)
         call prod(mxsize,psi,wk3)
      endif

      if (nel.lt.2) return

c     calculate first contributions from one particle operators and
c     Coulomb potential contributions within the same shell

      norb2=norb*norb

      vnt=zero
      vkt=zero
      woneel=zero
      wndc =zero

      do iorb=1,norb
         if (inhyd(iorb).eq.1) goto 10
         iborb=i1b(iorb)
         isiorb=i1si(iorb)
         nmut=i1mu(iorb)
         ibpot=i2b(iorb)	  
         isipot=i2si(iorb)
         oc=occ(iorb)
         
         isym=isymOrb(iorb)

c        calculate derivatives over mu and ni variables by means of matrix
c        multiplication

         call putin  (nni,nmut,isym,psi(iborb),wk3)
         call diffnu (nmut,wk3,wk0,wk1,wk2)
         call putout (nni,nmut,wk1,wk0)
         call diffmu (nmut,wk3,wk2)
         call putout (nni,nmut,wk0,wk2)

c        add derivatives over mu and ni

         call add (isiorb,wk0,wk1)

c        add contribution from phi part of laplasian
c        e enters the expression with minus sign which is already
c        incorporated in e

         if (mm(iorb).ne.0) then
	    w=dble(mm(iorb)*mm(iorb))
	    call copy (isiorb,e,ione,wk0,ione)
	    if (mm(iorb).ne.1) then
               call scal (isiorb,w,wk0,ione)
	    endif
	    call prod  (isiorb,psi(iborb),wk0)
	    call add   (isiorb,wk0,wk1)
         endif
         
         call copy (isiorb,wk1,ione,wk2,ione)
         call prod  (isiorb,psi(iborb),wk2)
         w=dot(isiorb,wgt1,ione,wk2,ione)
         vk(iorb)=w
         
         call copy (isiorb,f0,ione,wk0,ione)
         call prod  (isiorb,psi(iborb),wk0)
         
         call prod2 (isiorb,psi(iborb),wk0,wk2)
         w=dot(isiorb,wgt1,ione,wk2,ione)
         vn(iorb)=w

         call add	 (isiorb,wk0,wk1)
         call prod (isiorb,psi(iborb),wk1)
         
         w =dot(isiorb,wgt1,ione,wk1,ione)
         woneel=woneel+oc*w

         vnt=vnt+oc*vn(iorb)
         vkt=vkt+oc*vk(iorb)
 10      continue
      enddo

c     contribution from coulomb interaction within the same shell

c     calculate the coulomb potential contribution from all orbitals
c     (include 1/2 factor )

      do i=1,mxsize
         wk2(i)=zero
      enddo
      
      do iorb=1,norb
         if (inhyd(iorb).eq.1) goto 20
         ibpot=i2b(iorb)
         isipot=i2si(iorb)	
         oc=occ(iorb)/two
         call axpy (isipot,oc,pot(ibpot),ione,wk2,ione)
 20      continue
      enddo

c     contribution from the Coulomb interaction

      do iorb=1,norb
         if (inhyd(iorb).eq.1) goto 30
         iborb=i1b(iorb)
         isiorb=i1si(iorb)	  
         call prod2 (isiorb,psi(iborb),psi(iborb),wk0)
         call prod (isiorb,wk2,wk0)
         call scal (isiorb,occ(iorb),wk0,ione)
         w=dot(isiorb,wgt2,ione,wk0,ione)
         wndc=wndc+w
 30      continue
      enddo
      
c     DFT exchange energy corrections

      wex=0.d0
      wcorr=0.d0

      if     (idftex.eq.1) then
         wex=exxalpha(psi,wgt2,rhot,rhotup,rhotdown,
     &           grhot,grhotup,grhotdown,
     &           wk0,wk1,wk2,wk3,wk10,wk11,wk12,wk13)
c         call ehemsic(psi,pot,wgt2,wk1,wk0) 

      elseif (idftex.eq.2) then
         wex=exbe88(psi,wgt2,rhot,rhotup,rhotdown,
     &        grhot,grhotup,grhotdown,
     &        wk0,wk1,wk2,wk3,wk10,wk11,wk12,wk13)
      elseif (idftex.eq.3) then
         wex=expw86(psi,wgt2,rhot,rhotup,rhotdown,
     &        grhot,grhotup,grhotdown,
     &        wk0,wk1,wk2,wk3,wk10,wk11,wk12,wk13)
      elseif (idftex.eq.4) then
         wex=expw91(psi,wgt2,rhot,rhotup,rhotdown,
     &        grhot,grhotup,grhotdown,
     &        wk0,wk1,wk2,wk3,wk10,wk11,wk12,wk13)
      endif

c     DFT correlation energy corrections

      if (idftcorr.eq.1) then
         wcorr=eclyptot(psi,wgt2,rhot,rhotup,rhotdown,
     &           grhot,grhotup,grhotdown,
     &           wk0,wk1,wk2,wk3,wk10,wk11,wk12,wk13)
      elseif (idftcorr.eq.2) then
         wcorr=ecvwntot(psi,wgt2,rhot,rhotup,rhotdown,
     &           grhot,grhotup,grhotdown,
     &           wk0,wk1,wk2,wk3,wk10,wk11,wk12,wk13)
      endif

      evt=woneel+wndc+wex+wcorr

      engt(1)=evt
      etot=evt+z1*z2/r
      virrat=evt/vkt

      enkin=vkt
      ennucel=vnt
      encoul=wndc
      enexch=wex

      encouldft=wndc
      enexchdft=wex

      edftcorr=wcorr

      entot=enkin+ennucel+encoul+enexch+edftcorr+z1*z2/r

      if (iprint(77).ne.0) then
         write(*,*)'etotalDFT - woneel,wndc,wex,etotal ',
     &        woneel,wndc,wex,evt
      endif

      return
      end




***************************************************************************
*                                                                         *
*   Copyright (C) 2010 Jacek Kobus <jkob@fizyka.umk.pl>                   *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### ehemsic ###
c ### SOURCE: exbe88.f and exbe88sup.f
c     Calculates SIC according to Samal and Harbola
c             (J.Phys.B:At.Mol.Opt.Phys. 38 (2005) 3765-3777 
c					-- eqn. 19

      subroutine ehemsic (psi,pot,wgt2,wk1,wk0)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const23=2.d0/3.d0,const43=4.d0/3.d0,
     &     const53=5.d0/3.d0)
      include 'commons8.inc'

      dimension psi(*),pot(*),wgt2(*),wk1(*),wk0(*)

c **** E^{SIC}_i[\phi_i] for each orbital
      do i=1,mxsize
         wk0(i) = 0.d0
         wk1(i) = 0.d0
      enddo
      write(*,'(" ** WARNING ** [SIC-(1)] MULTIPLY BY ocup = ")')
c contrib.f
c     Coulomb potential contributions within the same shell
      esic1 = zero
      do iorb=1,norb
         iborb=i1b(iorb)
         isiorb=i1si(iorb)
         nmut=i1mu(iorb)
         ibpot=i2b(iorb)	  
         isipot=i2si(iorb)
c-         oc=occ(iorb)
c        contribution from coulomb interaction within the same shell      
c-        if (oc.gt.one) then
	    if (isipot.ne.isiorb) then
               write(iout6,*) 'coulomb potentials and orbitals',
     &              ' have to defined on the same number of subgrids'
               stop 'contrib'
            endif				
            call copy (isipot,pot(ibpot),ione,wk0,ione)
            call prod  (isiorb,psi(iborb),wk0)
            call prod  (isiorb,psi(iborb),wk0)
            w=dot(isiorb,wgt2,ione,wk0,ione)
c-            esic1=w*oc/two
            esic1=w/two
c-            wdcoul=wdcoul+oc*(oc-one)/2.d0*w
c-         endif	
      write(*,'("     SIC-1            = ",
     &                                     i3,f20.12)') iorb,esic1

      enddo

c-----------------
c **** E^{LSD}_x[\rho(\phi_i)] for each orbital
      CXLDA   = -(three/four)*(three/pii)**const13
      CXLSD   = CXLDA*two**const13

      do iksic=1,norb
      esic2 = zero
      do i=1,mxsize
         wk1(i)=0.d0
      enddo

      do iorb=1,norb
         iborb=i1b(iorb)
         isiorb=i1si(iorb)
         nmut=i1mu(iorb)
         
         if(iorb.EQ.iksic)THEN
           call prod2 (isiorb,psi(iborb),psi(iborb),wk1)
         endif
      enddo

      do i=1,mxsize
         wk1(i)=wk1(i)**const43
      enddo

c     take care of f4 factor
      call multf4(wk1)
      esic2 = CXLSD*dot(mxsize,wgt2,ione,wk1,ione)

      write(*,'("     SIC-2            = ",
     &                                     i3,f20.12)') iksic,esic2
      enddo

c-- ALTERNATE METHOD FOR \iint \frac{|\phi_i(r_1)|^2 * |\phi_i(r_1)|^2}
c--                                 {|r_1 - r_2|} dr_1 dr_2
c-- Same result as "wsic1"
c--      do iorb=1,norb
c--         ibpot=i2b(iorb)
c--         isipot=i2si(iorb)	
c--         IF(iorb.EQ.iksic)THEN
c--           oc=1.D0
c--         ELSE
c--           oc=0.D0
c--        ENDIF
c--         call axpy (isipot,oc,pot(ibpot),ione,wk1,ione)
c--      enddo

c     contribution from the Coulomb interaction
c--      wndc = 0.d0
c--      do iorb=1,norb
c--         iborb=i1b(iorb)
c--         isiorb=i1si(iorb)	
c--         IF(iorb.EQ.iksic)THEN  
c--         call prod2 (isiorb,psi(iborb),psi(iborb),wk0)
c--         call prod (isiorb,wk1,wk0)
c--         w=dot(isiorb,wgt2,ione,wk0,ione)
c--         wndc=wndc+w
c--         ENDIF
c--      enddo
c--      esic1 = wndc
c--      write(*,'("     Exchange [MLSD-SIC] (1)            = ",
c--     &                                     f20.12)') esic1

      return
      end


