***************************************************************************
*                                                                         *
*   Copyright (C) 2010 Jacek Kobus <jkob@fizyka.umk.pl>                   *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### expw86sup ###

c     Calculates exchange energy according to a formula of Parr and Wang Yue
c     Phys. Rev. B 54, 16 533 (1996), Phys. Rev. B 45, 13 244 (1992).

c     J. P. Perdew, J. A. Chevary, S. H. Vosko, K. A. Jackson, M. R. Pederson 
c     and C. Fiolhais, Phys. Rev. B 46, 6671 (1992)

      real*8 function expw91sup (wgt2,rho,grho,wk0,wk1)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const23=2.d0/3.d0,const43=4.d0/3.d0,
     &     const83=8.d0/3.d0)

      parameter(a=7.79560d0,a1=0.196450d0,a2=0.27430d0,a3=0.150840d0,
     &          a4=100.0d0,b1=0.0040d0)

      include 'commons8.inc'	
      dimension wgt2(*),rho(*),grho(*),wk0(*),wk1(*)

c arcsinh
      ash(s)=log(s+sqrt(one+s*s))

      const=(24.0d0*pii*pii)**(-const23)

c     grho = nabla rho  nabla rho
c     |nabla rho| = sqrt(grho)

      do i=1,mxsize
         if (abs(rho(i)).lt.precis) then
            wk0(i)=0.d0
         else
            rho43=rho(i)**const43
            s2=const*grho(i)/(rho43*rho43)
            s=sqrt(s2)
            wk0(i)=rho43*
     &           (one+a1*s*ash(a*s)+s2*(a2-a3*exp(-a4*s2)))/
     &           (one+a1*s*ash(a*s)+b1*s2*s2)
         endif
      enddo	  

c     take care of F4 factor
      call multf4(wk0)

      expw91sup=dot(mxsize,wgt2,ione,wk0,ione)

      return
      end


