***************************************************************************
*                                                                         *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### fermi ###

c     Analyzes Fermi charge distribution.

      subroutine fermi
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'

      if (iprint(230).ne.0) then
         write(*,*) ' '
         write(*,*) '  finite nuclei with the Fermi charge',
     &        ' distribution'
      endif

c     calculate parameters of the finite nucleus charge distribution
c     (Fremi distribution)

      fmtoau=1.0d-13/ainfcm

      if (z1.ne.0.d0) then

c        set atomic weight for centre A

         atw=z1atmass
         at3=atw**(1.d0/3.d0)
         rrmsfm = 0.836d0*at3+0.570d0
         tfm = 2.30d 00

c        change units from fm into bohr

         rrms = rrmsfm*fmtoau
         t = tfm*fmtoau  
         a = t/(4.0d0*log(3.0d0))         
         facto1 = rrms**2-(7.0d0/5.0d0)*(pii**2)*(a**2)
         c = sqrt (5.0d0/3.0d0) * sqrt (facto1) 

         ni=nni
         nc1=0
         xr2=r/2.0d0
         do mu=1,mxnmu
	    rr=sqrt(vxisq(mu)+vetasq(ni)-1.d0)
            xrr=xr2*rr
            if (abs(xrr-xr2).lt.c) then 
               nc1=nc1+1
               if (iprint(231).ne.0) write(*,*) 'mu,xrr',mu,xrr
            endif
         enddo

         mu=1
         nc2=0
         do ni=nni,nni/2,-1
	    rr=sqrt(vxisq(mu)+vetasq(ni)-1.d0)
            xrr=xr2*rr
            if (abs(xrr-xr2).lt.c) then
               nc2=nc2+1
               if (iprint(231).ne.0) write(*,*) 'ni,xrr',ni,xrr
            endif
         enddo

         if (iprt.ne.0) then
            write(*,*) '    center A: '
            write(*,*) '    c = ',c,' bohr', '  a = ',a,' bohr' 
            write(*,*) '    nu=-1:',nc1, ' points inside radius c'
            write(*,*) '    mu= 1:',nc2, ' points inside radius c'
            write(*,*) ' '
         endif
         elseif (z2.ne.0.d0) then

c           set atomic weight for centre B

         atw=z2atmass
         at3=atw**(1.d0/3.d0)
         rrmsfm = 0.836d0*at3+0.570d0
         tfm = 2.30d0

c        change units from fm into bohr

         rrms = rrmsfm*fmtoau
         t = tfm*fmtoau  
         a = t/(4.0d0*log (3.0d0))         
         facto1 = rrms**2-(7.0d0/5.0d0)*(pii**2)*(a**2)
         c = sqrt (5.0d0/3.0d0) * sqrt (facto1) 

         ni=1
         nc1=0
         xr2=r/2.0d0
         do mu=1,mxnmu
	    rr=sqrt(vxisq(mu)+vetasq(ni)-1.d0)
            xrr=xr2*rr
            if (abs(xrr-xr2).lt.c) then 
               nc1=nc1+1
               if (iprint(231).ne.0) write(*,*) 'mu,xrr',mu,xrr
            endif
         enddo

         mu=1
         nc2=0
         do ni=nni,nni/2,-1
	    rr=sqrt(vxisq(mu)+vetasq(ni)-1.d0)
            xrr=xr2*rr
            if (abs(xrr-xr2).lt.c) then
               nc2=nc2+1
               if (iprint(231).ne.0) write(*,*) 'ni,xrr',ni,xrr
            endif
         enddo

         if (iprint(230).ne.0) then
            write(*,*) '     Center B: '
            write(*,*) '     c = ',c,' bohr', '  a = ',a,' bohr' 
            write(*,*) '     nu=1:',nc1, ' points inside radius c'
            write(*,*) '     mu= 1:',nc2, ' points inside radius c'
         endif
      endif

      return
      end








