***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### flp ###  

c     It is assumed that function f(r) is tabulated at grid points r_i,
c     i=1,...,n which are stored in array r and the corresponding values
c     f_i are in array f.

c     The function fr0 uses the Lagrange polynomial of a given order to
c     calculate f(r0). The interpolation employs grid points adjacent to
c     r0.

c   iord - number of grid point used for interpolation 
c     n  - dimension of arrys r and f
c     r  - array of abscissas
c     f  - array of corresponding values of a given function f
c     r0 - an abscissa for which f(r0) is calculated via 8th-order 
c          Lagrange polynomial
 
      real*8 function flp(iord,n,r,f,r0)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      dimension r(n),f(n),a(12),coeff(9),coeff2(9,9)

c     iord value for 2th order (3-point) Lagrange polynomial
c     parameter (iord=3)

c     iord value for 8th order (9-point) Lagrange polynomial
c     parameter (iord=9)


c     find the smallest element of arry r that is greater than
c     r0. Whenever possible [iord/2] points to its left and right are
c     used to construct the interpolation polynomial (the start and the
c     end of r array are taken care of)

      nearest=1
      do i=1,n
         if (r(i).gt.r0) then
            nearest=i
            goto 10
         endif
      enddo
 10   continue

c     calculate coefficients of iord Lagrange polynomials (begining with
c     istart+1) and store them in coeff2

      istart=nearest-(iord/2+1)
      if (istart.lt.1) istart=0
      if (istart+iord.gt.n) istart=n-iord
      do k=1,iord
         call lpcoeff(iord,istart,k,r,coeff)
         do i=1,iord
            coeff2(i,k)=coeff(i)
         enddo
      enddo

c     evaluate the value of the Lagrange interpolation polynomial at r0

      flp=0.d0
      do k=1,iord
         flp=flp+f(istart+k)*vlpcoeff(iord,r0,coeff2(1,k))
      enddo
      return
      end


