***************************************************************************
*                                                                         *
*   Copyright (C) 2010 Jacek Kobus <jkob@fizyka.umk.pl>                   *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### fpw86 ###

c     Calculates exchange potential according to a formula of Perdew and
c     Wang Yue (PRB 33(12) (1986) 8800)

      subroutine fpw86 (psi,f4,rhot,rhotup,rhotdown,
     &     grhot,grhotup,grhotdown,wk0,wk1,wk2,wk3,wk4,
     &     wk5,wk6,wk7)

      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0,const43=4.d0/3.d0,
     &     const115=1.d0/15.d0)
      include 'commons8.inc'	

      dimension psi(*),f4(*),wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),
     &     wk5(*),wk6(*),wk7(*),rhot(*),rhotup(*),rhotdown(*),
     &     grhot(*),grhotup(*),grhotdown(*)

      
      do i=1,mxsize
         rhotup(i)   =0.d0
         rhotdown(i) =0.d0
      enddo	  
      
c     calculate total densities due to up and down spins
      do iorb=1,norb
         if (inhyd(iorb).eq.1) goto 10
         iborb=i1b(iorb)
         isiorb=i1si(iorb)
         nmut=i1mu(iorb)
         
         call exocc (iorb,ocup,ocdown)

         call prod2 (isiorb,psi(iborb),psi(iborb),wk1)
         call scal (isiorb,ocup,wk1,ione)
         
         call prod2 (isiorb,psi(iborb),psi(iborb),wk2)
         call scal (isiorb,ocdown,wk2,ione)
       
c        store total densities 
         call add(isiorb,wk1,rhotup)
         call add(isiorb,wk2,rhotdown)
 10      continue
      enddo

      call fpw86sup (rhotup,grhotup,rhot,
     &     wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7)

      call fpw86sup (rhotdown,grhotdown,rhot,
     &     wk0,wk1,wk2,wk3,wk4,wk5,wk6,wk7)

      call scal(mxsize,half,grhotup,ione)
      call scal(mxsize,half,grhotdown,ione)

c     add contributions due to up and down spins
      call copy(mxsize,grhotup,ione,wk2,ione)
      call add (mxsize,grhotdown,wk2)

c     take care of F4 factor
      call prod (mxsize,f4,wk2)
      return
      end



