***************************************************************************
*                                                                         *
*   Copyright (C) 2010 Jacek Kobus <jkob@fizyka.umk.pl>                   *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### fvwnsupcs ###

c     Calculates the correlation potential of VWN using the closed-shell
c     formula; it is returned in wk10 (see also fvwncs)

      subroutine fvwnsupcs (rhot,rhotup,rhotdown,wk0,wk1,wk2,wk3,wk4,
     &     wk5,wk6,wk7,wk8,wk9,wk10)

      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)

      parameter (const16=1.d0/6.d0,const18=1.d0/8.d0,
     &     const76=7.d0/6.d0,const56=5.d0/6.d0)
      parameter(ck1=0.0310907d0,cl1=-0.10498d0,
     &     cm1=3.72744d0,cn1=12.9352d0)

      include 'commons8.inc'	

      dimension wk0(*),wk1(*),wk2(*),wk3(*),wk4(*),
     &     wk5(*),wk6(*),wk7(*),wk8(*),wk9(*),wk10(*),
     &     rhot(*),rhotup(*),rhotdown(*)

c     total density in rhot

      constx=(three/(four*pii))**const16
      constxp=-(four*pii/three)**const56/(eight*pii)

c dx/d(rho)=xderrhot
      do i=1,mxsize
         if (abs(rhot(i)).lt.precis) then
            wk10(i)=0.d0
         else
            x=constx*rhot(i)**(-const16)
            xderrhot=constxp*rhot(i)**(-const76)
            g1=qvwn(x,ck1,cl1,cm1,cn1)
            g2=qvwnderx(x,ck1,cl1,cm1,cn1)
c            print *,'2',i,constx,constxp,x,xderrhot,g1,g2
            wk10(i)=g1+rhot(i)*xderrhot*g2
         endif
      enddo	  

      return
      end
