***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### hydrogenOrbA ###  
c
c     This routine returns an arrys containing a hydrogenic orbital
c     centred on centre A for a given set of n,l,m and Z. 

      subroutine hydrogenOrbA (orbital,n1,l1,m1,ez1)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      dimension orbital(*)

c     normalization factor for Laguere polynomials

      fn1=(2.d0*ez1/dble(n1))**(3.d0/2.d0+dble(l1))
     &     *sqrt(factor(n1+l1)/(2.d0*dble(n1)*factor(n1-l1-1)))
     &     /factor(2*l1+1)
      
c     normalization factor for spherical harmonics
      
      shn1=(-1.d0)**dble(m1)/sqrt(4.d0*pii)*
     &     sqrt((2*l1+1)*factor(l1-m1)/factor(l1+m1))
      if (m1.eq.0) shn1=1.d0/sqrt(4.d0*pii)*
     &     sqrt((2*l1+1)*factor(l1-m1)/factor(l1+m1))
      
c          multiplication factor fot the associate Legendre polynomials
c     alp1=(-1.d0/2.d0)**m1*factor(l1+m1)/factor(l1-m1)/factor(m1)
      
      do in=1,nni
         do imu=1,mxnmu
            inioff=(imu-1)*nni
            igp=inioff+in

c           for each grid point, e.i. for (vmu(imu),vni(ini)) 
c           determine its distance |_r1| and |_r2| from the nuclei A 
c           and B and cosine of the polar angles costh1 and costh2 
c           between z axis and the vectors _r1 and _r2
c
c           rr=sqrt(xi(j,6)-eta(i,3)+1.d0)
c           rr=sqrt(vxi12(j)-veta12(i)+1.d0)	
c           rr=sqrt(vxisq(j)+vetasq(i)-1.d0)	
c           costh=eta(i,1)*xi(j,4)/rr
            
            rr=(r/2.d0)*sqrt(vxisq(imu)+vetasq(in)-1.d0)		
            z=(r/2.d0)*vxi(imu)*veta(in)
            r1t=(r/2.d0)*(vxi(imu)+veta(in))

c           if (r1t.lt.precis) then
            if (r1t.eq.zero) then
c              jk 02/01
               costh1=-1.d0
c              costh1=0.d0
c              r1t=1.0d-10
c              psi1=0.d0
c              psi2=0.d0
c              goto 99
            else   
               costh1=(z+r/2.d0)/r1t
            endif
            
c           calculate radial part of the hydrogenic orbital centered on
c           both the nuclei
            
            orbital(igp)=fn1*plaguer(n1,l1,ez1,r1t)
     &           *shn1*plegendg(l1,m1,costh1)
         enddo
      enddo

      return
      end
