***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### mpoleMom ###
c
c     Recalculates multipole moment expansion coefficients every time
c     demax(1), i.e. maximum error in orbital energy, is reduced by
c     facmul. Multipole moments used to calculate asymptotic values of
c     Coulomb and exchange potentials are stored in cmulti and
c     exc(di|qu|oc|he|5-8) arrays, respectively.

c     Coefficients and then asymptotic values are recalculated only for
c     orbitals which undergo relaxation, i.e. those being touched
c     (itouch=1).

      subroutine mpoleMom (cw_orb,cw_suppl,cw_sctch)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      dimension cw_orb(*),cw_suppl(*),cw_sctch(*)

      if (nel.eq.1) return

      call getCpuTime(time1)
      call coulMom (cw_orb,cw_suppl(i4b(9)),cw_suppl(i4b(14)),
     &     cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),cw_sctch(i5b( 3)),
     &     cw_sctch(i5b( 4)),cw_sctch(i5b( 5)),cw_sctch(i5b( 6)),     
     &     cw_sctch(i5b( 7)),
     &     cw_sctch(i5b( 8)),cw_sctch(i5b( 9)),cw_sctch(i5b(10)))

      if (nel.gt.1.and.imethod.eq.1) then
         do  iorb1=1,norb
            do  iorb2=iorb1,norb
               
               call exchMom (iorb1,iorb2,cw_orb,
     &              cw_suppl(i4b(9)),cw_suppl(i4b(14)),	     
     &              cw_sctch(i5b( 1)),cw_sctch(i5b( 2)),
     &              cw_sctch(i5b( 3)),cw_sctch(i5b( 4)),
     &              cw_sctch(i5b( 5)),cw_sctch(i5b( 6)),     
     &              cw_sctch(i5b( 7)),cw_sctch(i5b( 8)),
     &              cw_sctch(i5b( 9)),cw_sctch(i5b(10)))
            enddo
         enddo
      endif
      call getCpuTime (time2)
      tmomen =tmomen + (time2-time1)
      if (iprtlev.ne.3) then 
         write(iout6,*) 
     &        '... multipole moment expansion coefficients',
     &        ' recalculated ...'
      endif

      return
      end


