***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### mulex ###

c     This routine generates values of the associate Legendre functions
c     P(k,q) multiplied by [(k-|q|)!/(k+|q|)!]^{1/2} for k=1,..,8 and
c     q=mt.  See also routine which employs the same definition of the
c     functions multi.

c     Note that the original routine (mulex) generates values scaled by
c     (-1)**mt

      subroutine mulex(i,j,mt,dome)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      dimension dome(10)

      dome(1)=0.d0

      if (mt.gt.4) return

      rr=sqrt(vxisq(j)+vetasq(i)-1.d0)		
      costh=0.d0
      if (abs(rr).gt.precis) then
         costh=veta(i)*vxi(j)/rr	 
      endif

      
      if (mt.eq.0) then
c        m=0
         dome(1)=costh
         dome(2)=(3.d0*costh*costh-1.d0)*0.5d0
         do n=2,mpole-1
            dome(n+1)=(dble(2*n+1)*costh*dome(n)-dble(n)*
     &           dome(n-1))/dble(n+1)
         enddo
         return

      elseif (mt.eq.1) then
c        m=1

         costh2=costh*costh
         sini2=abs(1.d0-costh2)
         sini =sqrt(sini2)
c        P_1^1
         dome(1)=-sini/sqrt(2.d0)
c        P_2^1
         dome(2)=-3.d0*sini*costh/sqrt(6.d0)

         if (mpole.lt.3) return
c        P_3^1
         dome(3)=(3.d0/2.d0)*sini*(1.d0-5.d0*costh2)/sqrt(12.d0)

         if (mpole.lt.4) return
c        P_4^1
         dome(4)=(5.d0/2.d0)*sini*costh
     &        *(3.d0-7.d0*costh2)/sqrt(20.d0)

         if (mpole.lt.5) return
c        P_5^1
         costh4=costh2*costh2
         dome(5)=(15.d0/8.d0)*sini
     &        *(-1.d0+14.d0*costh2-21.d0*costh4)/sqrt(30.d0)
         if (mpole.lt.6) return
c        P_6^1
         dome(6)=(21.d0/8.d0)*sini*costh
     &        *(-5.d0+30.d0*costh2-33.d0*costh4)/sqrt(42.d0)

         if (mpole.lt.7) return
c        P_7^1
         costh6=costh4*costh2
         dome(7)=(7.d0/16.d0)*sini
     &        *(5.d0-135.d0*costh2+495.d0*costh4
     &        -429.d0*costh6)/sqrt(56.d0)

         if (mpole.lt.8) return
c        P_8^1
         dome(8)=(9.d0/16.d0)*sini*costh*(35.d0-385.d0*costh2
     &        +1001.d0*costh4-715.d0*costh6)/sqrt(72.d0)
         return

      elseif (mt.eq.2) then
c        m=2
         costh2=costh*costh
         sini2=abs(1.d0-costh2)

c        P_1^2
         dome(1)=0.d0
c        P_2^2
         dome(2)=3.d0*sini2/sqrt(24.d0)

         if (mpole.lt.3) return
c        P_3^2
         dome(3)=15.d0*costh*sini2/sqrt(120.d0)

         if (mpole.lt.4) return
c        P_4^2
         costh4=costh2*costh2
         dome(4)=(15.d0/2.d0)*(-1.d0+8.d0*costh2
     &        -7.d0*costh4)/sqrt(360.d0)

         if (mpole.lt.5) return
c        P_5^2
         dome(5)=(105.d0/2.d0)*costh*(-1.d0+4.d0*costh2
     &        -3.d0*costh4)/sqrt(840.d0)

         if (mpole.lt.6) return
c        P_6^2
         costh6=costh4*costh2
         dome(6)=(105.d0/8.d0)
     &        *(1.d0-19.d0*costh2+51.d0*costh4
     &        -33.d0*costh6)/sqrt(1680.d0)

         if (mpole.lt.7) return
c        P_7^2
         dome(7)=(63.d0/8.d0)*costh
     &        *(15.d0-125.d0*costh2+253.d0*costh4
     &        -143.d0*costh6)/sqrt(3024.d0)

         if (mpole.lt.8) return
c        P_8^2
         costh8=costh6*costh2
         dome(8)=(315.d0/8.d0)*(-1.d0+34.d0*costh2-176.d0*costh4
     &        +286.d0*costh6-143.d0*costh8)/sqrt(5040.d0)
         return

      elseif (mt.eq.3) then 
c        m=3

         costh2=costh*costh
         sini2=abs(1.d0-costh2)
         sini =sqrt(sini2)
         sini3=sini2*sini

         dome(1)=0.d0
         dome(2)=0.d0

         if (mpole.lt.3) return
c        P_3^3
         dome(3)=-15.d0*sini3/sqrt(720.d0)

         if (mpole.lt.4) return
c        P_4^3
         dome(4)=-105.d0*sini3*costh/sqrt(5040.d0)

         if (mpole.lt.5) return
c        P_5^3
         dome(5)=-(105.d0/2.d0)*sini3
     &        *(-1.d0+9.d0*costh2)/sqrt(20160.d0)

         if (mpole.lt.6) return
c        P_6^3
         dome(6)=-(315.d0/2.d0)*sini3*costh
     &        *(-3.d0+11.d0*costh2)/sqrt(60480.d0)

         if (mpole.lt.7) return
c        P_7^3
         costh4=costh2*costh2
         dome(7)=-(315.d0/8.d0)*sini3
     &        *(3.d0-66.d0*costh2+143.d0*costh4)/sqrt(151200.d0)

         if (mpole.lt.8) return
         dome(8)=-(3465.d0/8.d0)*sini3*costh
     &        *(3.d0-26.d0*costh2+39.d0*costh4)/sqrt(332640.d0)
         return

      elseif (mt.eq.4) then
c        m=4

         dome(1)=0.d0
         dome(2)=0.d0
         dome(3)=0.d0

         if (mpole.lt.4) return
c        P_4^4
         costh2=costh*costh
         costh4=costh2*costh2
         dome(4)=105.d0*(1.d0-2.d0*costh2+costh4)/sqrt(40320.d0)

         if (mpole.lt.5) return
c        P_5^4
         dome(5)=945.d0*costh*(1.d0-2.d0*costh2+costh4)/sqrt(362880.d0)

         if (mpole.lt.6) return
c        P_6^4
         costh6=costh4*costh2
         dome(6)=(945.d0/2.d0)
     &        *(-1.d0+13*costh2-23.d0*costh4
     &        +11.d0*costh6)/sqrt(1814400.d0)

         if (mpole.lt.7) return
c        P_7^4
         dome(7)=(3465.d0/2.d0)*costh
     &        *(-3.d0+19*costh2-29.d0*costh4
     &        +13.d0*costh6)/sqrt(6652800.d0)

         if (mpole.lt.8) return
c        P_8^4
         costh8=costh6*costh2
         dome(8)=(10395.d0/8.d0)*(1.d0-28.d0*costh2+118.d0*costh4
     &        -156.d0*costh6+65.d0*costh8)/sqrt(19958400.d0)
         return
      endif

      return	
      end

