***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### putin ###

c     Immerses FUN array into WORK and adds boundary values from symmetry

c     Immerses FUN array into WORK and provides missing values along (1,i), (nni,i) and (j,1) 
c     boundaries according to orbital symmetry isym.
c     Values along (j,nmi) boundary are set to zero as orbitals decay exponentially in this region.

      subroutine putin (nni,nmi,isym,fun,work)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)

      dimension fun(nni,nmi),work(nni+8,nmi+8)

c     fill the interior of work array

      do i=1,nmi
         do j=1,nni
            work(j+4,i+4)=fun(j,i)
         enddo
      enddo

c     isym = 1 - even symmetry, isym =-1 - odd symmetry
c     values over i=nmi bondary are determined from the asymptotic expansion

c     mu=nmu+5....nmu+8
c     the following code is necessary since the derivatives must be 
c     calculated up to i=nmi

      if (isym.eq.1) then

c  mu=1...4	
         do i=2,5
            do j=1,nni
               work(j+4,6-i)= fun(j,i)
            enddo
         enddo
         
c        ni=1...4
         do i=1,nmi
            do j=2,5
               work(6-j,i+4)= fun(j,i)
            enddo
         enddo

c        ni=ni+4...ni+8
	  do i=1,nmi
             jj=0
             do j=nni-4,nni-1
                jj=jj+1
                work(nni+9-jj,i+4)= fun(j,i)
             enddo
          enddo
      else
         do i=2,5
            do j=1,nni
               work(j+4,6-i)=-fun(j,i)
            enddo
         enddo

         do i=1,nmi
            do j=2,5
               work(6-j,i+4)=-fun(j,i)
            enddo
         enddo

         do i=1,nmi
	    jj=0
            do j=nni-4,nni-1
               jj=jj+1
               work(nni+9-jj,i+4)=-fun(j,i)
            enddo
         enddo
      endif

      return
      end
