***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### radialden ###

c     Calculates total radial densities relative to centres A (z-R/2)
c     and B (z+R/2) along the internuclear axis (from A to -\infty or B
c     to +\infty).

      subroutine radialden(psi,wk0)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (const13=1.d0/3.d0)
      include 'commons8.inc'	
      dimension psi(*),wk0(*)

      ngorb=i1si(norb)
      call zeroArray(ngorb,wk0)
	 	       
      do iorb=1,norb
         iborb=i1b(iorb)	   	 
         coo=occ(iorb)
         do i=1,ngorb
            wk0(i)=wk0(i)+coo*psi(iborb+i-1)*psi(iborb+i-1)
         enddo
      enddo

      iunit=99
      if (iprint(110).eq.1) then

c        print radial density relative to centre A along the
c        internuclear axis (-R_{\infty}<=z<=-R/2)
 
         open(iunit,file='density-A',status='unknown',form='formatted')
         call prtdenA(nni,i1mu(1),wk0,iunit)
         close(iunit)
      endif

      if (iprint(111).eq.1) then

c        print radial density relative to centre B along the
c        internuclear axis (R/2<=z<=R_{\infty})

         open(iunit,file='density-B',status='unknown',form='formatted')
         call prtdenB(ione,i1mu(1),wk0,iunit)
         close(iunit)
      endif

      return
      end

c ### prtdenA ###

      subroutine prtdenA (m,n,a,ioutmat)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      dimension a(m,n)

      write(ioutmat,'(10x,"r(au)",12x,"total electronic density")')
      in=m
      do imu=1,n
         r1t=(half*r)*(vxi(imu)+veta(in))
         write(ioutmat,'(2E25.16)') r1t,a(in,imu)
      enddo

      return
      end

c ### prtdenB ###

      subroutine prtdenB (m,n,a,ioutmat)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      dimension a(m,n)

      write(ioutmat,'(10x,"r(au)",12x,"total electronic density")')
      in=1
      do imu=1,n
         r2t=(r/2.d0)*(vxi(imu)-veta(in))
         write(ioutmat,'(2E25.16)') r2t,a(in,imu)
      enddo

      return
      end
