***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### setDefaults ###
c
c     This routine sets default values of various (scalar and array)
c     variables.

      subroutine setDefaults
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'

c     no=isum=norb
c     iexd=maxorb
c     bond=orbsym

      inpform=0
      ioutform=0
      inout32=0
      inout64=0
      inout128=0

c     2dhf_input.dat is assumed
      idat=0


c     default data input format depends on compilation options
      lengthintin=lengthint
      lengthfpin=lengthfp

c     set format of floating-point numbers
      if (lengthfpin.eq.8) then
         formfp=formfp64
      else
         formfp=formfp128
      endif   

c     printout according to the precision used
      if (precis.gt.1.d-20) then
         iprint16=0
      else
         iprint16=1
      endif

c  initialization of some variables

c     if |z1-z2|<homolevl then the case is treated as a homonuclear one 
c     (ihomon=2) 

      homolevl=1.0d-6
      ihomon=0
      ibreak=0
      iinterp=0
     
      cutorb =1.d-6
      cutcoul=1.d-6
      cutexch=1.d-6
      iftail  =13    

c     diver  -  if demax(1) in proce is greater than diver
c               divergence in the scf process is detected and
c               the program stops

      diver = 1.d20   

c     default canonicalization coefficients

      icanon=1
      
c     multipol default value; the order of multipole expansion cannot
c     exceed maxmpole-1 (and mpole maxmpole)

      mpole=4

c     default number of grids and the default ordering of mesh points
      ngrids=1
      iorder(ngrids)=2
      ialtsweeps=0

c     scf default values
      
      maxscf  = 1000
      nobckup =  20
      ienterm =  10
      inoterm =  10
      iprtlev =   2

c     fix default values

      iexlorb  = 0
      iexlcoul = 0
      iexlexp  = 2
   
      exlorb  = 0.d0
      exlcoul = 0.d0
      exlexp  = 2.d0


c     HF method is chosen as a default (card METHOD is not compulsory any more)
      imethod=1
c     and DFT approach is switched off
      idft=0
      idftex=0
      idftcorr=0
      alphaf = two/three
      iscmc=0


c initials orbitals and potentials
      inclorb = 1
      iform=3
      ini4=0


c     sor default value
      maxsor1 = 1
      maxsor2 =10
      maxsor3 =10
      lsor    = 1
      ipoiss  = 1
      iomega  = 0
c     
      ovforb(1)=-one
      ovfcoul(1)=-one

c     scaling factor for omega value for orbitals and potentials
      omegasf=0.98d0

c     values based on test performed on FH 157x229/45
      omegasfOrb=0.979
      omegasfPot=0.996


c     skip nscf2skip iterations before starting the search for
c     saturation; examine the last nenlast and nnolast iterations in case
c     of orbital energy and normalization, respectively

      nscf2skip=600
      nenlast=20
      nnolast=20

c     ipot default value (point nuclei)

      ipot=0
      ifermi=0
      
c     plot default value (no data exported for making plots)

      iplot=0
      
c     additional relaxation of the least relaxed orbital is switched off
c     facrel is no longer used	

      facrel=-10.d0

c     search for optimal value of the overrelaxation parameter is
c     switched off
      
      search=1.d-15
      omincr=0.0d0
      
      ilagra=0
      sflagra=1.d0
      dflagra=0.d0

      nonortho=0

      ifefield=0
      ffield =0.d0
      fgrad  =0.d0
      zcutoff=1.d5
      iharm2xy=0
      magfield=0
      cmass =0.d0
      cmass1=0.d0

c     default order of interpolation polynomials when changing grids 
c     for orbitals and potentials

      iord_nu_orb=5
      iord_nu_coul=5
      iord_nu_exch=5
      
      iord_mu_orb=5
      iord_mu_coul=5
      iord_mu_exch=5

c     array iscforder is used to change the defaukt order of relaxation
c     of orbitals during scf process

      do i=1,maxorb
         iscforder(i)=0
      enddo

      incrni=10
      incrmu=10

      do i=1,maxorb
         gut(i)  ='        '
         bond(i) ='        '
      enddo
      do i=1,4*maxorb
         spin(i) ='        '
      enddo

      do i=1,2*maxorb
         lagraon(i)=0
      enddo


      do i=1,1000
         idbg(i)=0
         iprint(i)=0
      enddo

      do i=1,maxorb
         do j=1,maxorb
            nlmf(i,j)=0
            nonorthog(i,j)=0
         enddo
      enddo



c     if facmul>0 multipole moment expansion coefficients are
c     recalculated every time denmax, i.e. maximum error in orbital
c     energy, changes by the factor facmul if facmul<0 these
c     coefficients are not calculated

      facmul=1.15d0

c     set maximum allowed unit number
      call setmaxunit()

      
      return
      end
