***************************************************************************
*                                                                         *
*   Copyright (C) 2010 Jacek Kobus <jkob@fizyka.umk.pl>                   *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### setomega ###

c     Determines optimal omega overrelaxation parameter for relaxing
c     potentials according to a semiempirical formula due to B.Sobczak's
c     MSc thesis, Torun 2002

      function setomega()
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (a=0.603,b=0.79)
      include 'commons8.inc'

      rho=0.5*(cos(pii/dble(mxnmu))+cos(pii/dble(nni)))
      setomega=2.d0*a/(1.d0+sqrt(1.d0-rho*rho))+b
      return
      end

c     gnuplot fit to omega_orb  (FH)
c        a =  1.07272         +/- 0.01652      (1.54%)
c        b = -0.145639        +/- 0.03261      (22.39%)

c     gnuplot fit to omega_orb  (LiH)
c        a = 1.26598          +/- 0.04347      (3.433%)
c        b = -0.528979        +/- 0.08577      (16.21%)

      function setOmegaOrb()
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (a=1.073,b=-0.1456)
c      parameter (a=1.266,b=-0.5290)
      include 'commons8.inc'

      rho=0.5*(cos(pii/dble(mxnmu))+cos(pii/dble(nni)))
      setOmegaOrb=2.d0*a/(1.d0+sqrt(1.d0-rho*rho))+b
      return
      end


c     gnuplot fit to omega_pot  (FH)
c        a = 0.568667         +/- 0.007234     (1.272%)
c        b = 0.863499         +/- 0.01428      (1.654%)

c     gnuplot fit to omega_pot  (LiH)
c        a = 0.52864          +/- 0.005446     (1.03%)
c        b = 0.942668         +/- 0.01075      (1.14%)

      function setOmegaPot()
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (a=0.5687,b=0.8635)
c      parameter (a=0.5286,b=0.9427)
      include 'commons8.inc'

      rho=0.5*(cos(pii/dble(mxnmu))+cos(pii/dble(nni)))
      setOmegaPot=2.d0*a/(1.d0+sqrt(1.d0-rho*rho))+b

      return
      end


c     optimal omega for 2nd-order stencil
      function setOmegaPot2()
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      parameter (a=1.0,b=0.0)
      include 'commons8.inc'

      rho=0.5*(cos(pii/dble(mxnmu))+cos(pii/dble(nni)))
      setOmegaPot2=2.d0*a/(1.d0+sqrt(1.d0-rho*rho))+b
      return
      end




