***************************************************************************
*                                                                         *
*   Copyright (C) 1996 Leif Laaksonen, Dage Sundholm                      *
*   Copyright (C) 1996-2010 Jacek Kobus <jkob@fizyka.umk.pl>              *
*                                                                         *     
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License version 2 as     *
*   published by the Free Software Foundation.                            *
*                                                                         *
***************************************************************************
c ### writeDisk ### 

c     Writes orbitals, potenials, Lagrange multipliers (diagonal 
c     and off-diagonal) and multipole expansion coefficients to a disk 
c     file in either formatted or unformatted form

      subroutine writeDisk (cw_orb,cw_coul,cw_exch)
      implicit integer*4 (i-n)
      implicit real*8 (a-h,o-z)
      include 'commons8.inc'
      character*1 dtarr1(80),dtarr2(80)
      equivalence(header,dtarr1(1))
      equivalence(datetime,dtarr2(1))
      data i4,i8,i16/4,8,16/
 
c fixed precision
      integer*4 i4tmp(1),r8l60(1),r8l1200(1),r8l3660(1),r8mxsize(1)
      integer*4 i8tmp(1),r16l60(1),r16l1200(1),r16l3660(1),r16mxsize(1)

      dimension cw_orb(*),cw_coul(*),cw_exch(*)  

      rewind(iout24)
      write(iout24,'(a80)') header
      write(iout24,'(a80)') datetime
      
      write(iout24,formint) ngrids,nni,nmu
      write(iout24,formfp64) r,rgrid
      write(iout24,formfp64) z1,z2	
      write(iout24,formint) norb,nel,nexch

      call flush(iout24)

      if (inout32.eq.1) then
         li4tmp=10*maxorb+5*maxorb*(maxorb+1)/2
         call memAlloc(i4tmp,   li4tmp, i4, ioffset1)
         call memAlloc(r8l60,   maxorb, i8, ioffset2)
         call memAlloc(r8l1200, 20*maxorb, i8, ioffset3)
         call memAlloc(r8l3660, maxorb*(maxorb+1), i8, ioffset4)
         call memAlloc(r8mxsize,nni*mxnmu, i8, ioffset5)
         call wtdisk32 (cw_orb,cw_coul,cw_exch,
     &        i4tmp(ioffset1),r8l60(ioffset2),r8l1200(ioffset3),
     &        r8l3660(ioffset4),r8mxsize(ioffset5))

         call memDealloc(i4tmp,   li4tmp, i4, ioffset1)
         call memDealloc(r8l60,   maxorb, i8, ioffset2)
         call memDealloc(r8l1200, 20*maxorb, i8, ioffset3)
         call memDealloc(r8l3660, maxorb*(maxorb+1), i8, ioffset4)
         call memDealloc(r8mxsize,nni*mxnmu, i8, ioffset5)

      elseif (inout32.eq.2) then
         li4tmp=10*maxorb+5*maxorb*(maxorb+1)/2
         call memAlloc(i4tmp,   li4tmp, i4, ioffset1)
         call memAlloc(r8l60,   maxorb, i8, ioffset2)
         call memAlloc(r8l1200, 20*maxorb, i8, ioffset3)
         call memAlloc(r8l3660, maxorb*(maxorb+1), i8, ioffset4)
         call memAlloc(r8mxsize,nni*mxnmu, i8, ioffset5)
         call wtdisk32f (cw_orb,cw_coul,cw_exch,
     &        i4tmp(ioffset1),r8l60(ioffset2),r8l1200(ioffset3),
     &        r8l3660(ioffset4),r8mxsize(ioffset5))

         call memDealloc(i4tmp,   li4tmp, i4, ioffset1)
         call memDealloc(r8l60,   maxorb, i8, ioffset2)
         call memDealloc(r8l1200, 20*maxorb, i8, ioffset3)
         call memDealloc(r8l3660, maxorb*(maxorb+1), i8, ioffset4)
         call memDealloc(r8mxsize,nni*mxnmu, i8, ioffset5)

      elseif (inout64.eq.1) then
         li8tmp=10*maxorb+5*maxorb*(maxorb+1)/2
         call memAlloc(i8tmp,   li8tmp, i8, ioffset1)
         call memAlloc(r8l60,   maxorb, i8, ioffset2)
         call memAlloc(r8l1200, 20*maxorb, i8, ioffset3)
         call memAlloc(r8l3660, maxorb*(maxorb+1), i8, ioffset4)
         call memAlloc(r8mxsize,nni*mxnmu, i8, ioffset5)
         call wtdisk64 (cw_orb,cw_coul,cw_exch,
     &        i8tmp(ioffset1),r8l60(ioffset2),r8l1200(ioffset3),
     &        r8l3660(ioffset4),r8mxsize(ioffset5))

         call memDealloc(i8tmp,   li8tmp, i8, ioffset1)
         call memDealloc(r8l60,   maxorb, i8, ioffset2)
         call memDealloc(r8l1200, 20*maxorb, i8, ioffset3)
         call memDealloc(r8l3660, maxorb*(maxorb+1), i8, ioffset4)
         call memDealloc(r8mxsize,nni*mxnmu, i8, ioffset5)

      elseif (inout64.eq.2) then
         li8tmp=10*maxorb+5*maxorb*(maxorb+1)/2
         call memAlloc(i8tmp,   li8tmp, i8, ioffset1)
         call memAlloc(r8l60,   maxorb, i8, ioffset2)
         call memAlloc(r8l1200, 20*maxorb, i8, ioffset3)
         call memAlloc(r8l3660, maxorb*(maxorb+1), i8, ioffset4)
         call memAlloc(r8mxsize,nni*mxnmu, i8, ioffset5)
         call wtdisk64f (cw_orb,cw_coul,cw_exch,
     &        i8tmp(ioffset1),r8l60(ioffset2),r8l1200(ioffset3),
     &        r8l3660(ioffset4),r8mxsize(ioffset5))

         call memDealloc(i8tmp,   li8tmp, i8, ioffset1)
         call memDealloc(r8l60,   maxorb, i8, ioffset2)
         call memDealloc(r8l1200, 20*maxorb, i8, ioffset3)
         call memDealloc(r8l3660, maxorb*(maxorb+1), i8, ioffset4)
         call memDealloc(r8mxsize,nni*mxnmu, i8, ioffset5)

      elseif (inout128.eq.1) then
         li8tmp=10*maxorb+5*maxorb*(maxorb+1)/2
         call memAlloc(i8tmp,   li8tmp, i8, ioffset1)
         call memAlloc(r16l60,   maxorb, i16, ioffset2)
         call memAlloc(r16l1200, 20*maxorb, i16, ioffset3)
         call memAlloc(r16l3660, maxorb*(maxorb+1), i16, ioffset4)
         call memAlloc(r16mxsize,nni*mxnmu, i16, ioffset5)

         call wtdisk128 (cw_orb,cw_coul,cw_exch,
     &        i8tmp(ioffset1),r16l60(ioffset2),r16l1200(ioffset3),
     &        r16l3660(ioffset4),r16mxsize(ioffset5))

         call memDealloc(i8tmp,   li8tmp, i8, ioffset1)
         call memDealloc(r16l60,   maxorb, i16, ioffset2)
         call memDealloc(r16l1200, 20*maxorb, i16, ioffset3)
         call memDealloc(r16l3660, maxorb*(maxorb+1), i16, ioffset4)
         call memDealloc(r16mxsize,nni*mxnmu, i16, ioffset5)


      elseif (inout128.eq.2) then
         li8tmp=10*maxorb+5*maxorb*(maxorb+1)/2
         call memAlloc(i8tmp,   li8tmp, i8, ioffset1)
         call memAlloc(r16l60,   maxorb, i16, ioffset2)
         call memAlloc(r16l1200, 20*maxorb, i16, ioffset3)
         call memAlloc(r16l3660, maxorb*(maxorb+1), i16, ioffset4)
         call memAlloc(r16mxsize,nni*mxnmu, i16, ioffset5)

         call wtdisk128f (cw_orb,cw_coul,cw_exch,
     &        i8tmp(ioffset1),r16l60(ioffset2),r16l1200(ioffset3),
     &        r16l3660(ioffset4),r16mxsize(ioffset5))

         call memDealloc(i8tmp,   li8tmp, i8, ioffset1)
         call memDealloc(r16l60,   maxorb, i16, ioffset2)
         call memDealloc(r16l1200, 20*maxorb, i16, ioffset3)
         call memDealloc(r16l3660, maxorb*(maxorb+1), i16, ioffset4)
         call memDealloc(r16mxsize,nni*mxnmu, i16, ioffset5)
      else
         call wtdisknat(cw_orb,cw_coul,cw_exch)
      endif

      return
      end








